#!/usr/local/bin/python3

#
# Copyright (C) 2022 Jonathan Certes
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program. If not, see <http://www.gnu.org/licenses/>.
#

import pynusmv
import pynusmv.nusmv.parser.psl.psl

#===============================================================================

##
# \brief
#   Creates a list of NuSMV node pointers that all share the same type
#   attribute.
#
# \details
#   Recursively accesses NuSMV nodes from one specific node and compares their
#   type: if it matches the expected type, adds them to the list that is given
#   as a parameter. Stops recursively accessing nodes once the type cannot have
#   a hierarchy.
#
# \returns void
#
def recursivelyGetAllNodes(
  theTopNode,   #!< the node to get in the hierarchy from
  theNodeType,  #!< the type of nodes to add to the list
  theList       #!< the list to append
):
  if ( theTopNode.type == theNodeType ):
    list.append(theList, theTopNode)
  else:
    if theTopNode.type not in {
      pynusmv.nusmv.parser.parser.NUMBER_SIGNED_WORD,
      pynusmv.nusmv.parser.parser.NUMBER_UNSIGNED_WORD,
      pynusmv.nusmv.parser.parser.NUMBER_FRAC,
      pynusmv.nusmv.parser.parser.NUMBER_EXP,
      pynusmv.nusmv.parser.parser.NUMBER_REAL,
      pynusmv.nusmv.parser.parser.NUMBER,
      pynusmv.nusmv.parser.parser.ATOM,
      pynusmv.nusmv.parser.parser.FAILURE
    }:
      if ( pynusmv.nusmv.node.node.car(theTopNode) is not None ):
        recursivelyGetAllNodes( pynusmv.nusmv.node.node.car(theTopNode),
                                theNodeType,
                                theList
                              )
      if ( pynusmv.nusmv.node.node.cdr(theTopNode) is not None ):
        recursivelyGetAllNodes( pynusmv.nusmv.node.node.cdr(theTopNode),
                                theNodeType,
                                theList
                              )

#===============================================================================

pynusmv.init.init_nusmv()
pynusmv.nusmv.parser.parser.Parser_ReadSMVFromFile("spec.smv")

theTree = pynusmv.nusmv.parser.parser.cvar.parsed_tree
theList = list()
recursivelyGetAllNodes(theTree, pynusmv.nusmv.parser.parser.PSLSPEC, theList)
theList.reverse()

for theSpecNode in theList:
  # separate the name from the property:
  theNameNode = pynusmv.nusmv.node.node.cdr(theSpecNode)
  thePslNode  = pynusmv.nusmv.node.node.car(theSpecNode)
  # convert PSL property to LTL:
  theLtlNode = pynusmv.nusmv.parser.psl.psl.PslNode_convert_psl_to_core(thePslNode)
  #
  thePslString = pynusmv.nusmv.node.node.sprint_node(thePslNode)
  theLtlString = pynusmv.nusmv.node.node.sprint_node(theLtlNode)
  print(thePslString)
  print(" <-> " + theLtlString)
  print("")
